/* $Id: pca200e.h,v 1.7 1998/01/05 18:42:17 ud3 Exp $ */

#ifndef _PCA200E_h
#define _PCA200E_h

#include <linux/atmdev.h>

#define PCA200E_DUMP     _IOW('a', ATMIOC_SARPRV+0x1f, struct atmif_sioc)

#define DEV_LABEL "PCA-200E: "

#define PCA200E_HARDWARE_BASE_LENGTH	0x00200000
#define PCA200E_HCR_OFFSET		0x00100000 /* Board Control Register */
#define PCA200E_IMR_OFFSET		0x00100004 /* Host IRQ Mask Register */
#define PCA200E_PSR_OFFSET		0x00100008 /* PCI Specific  Register */
#define PCA200E_MON960_OFFSET		0x00000400 /* MON960 interface base  */
#define PCA200E_MON960_UART_FREE	0x00000000
#define PCA200E_MON960_UART_AVAIL	0x01000000
#define PCA200E_COMMON_ORIGIN		0x00004d40 /* who did define this ?  */
#define MAX_PDU_LENGTH			65535

/* buffer sizes should be a multiple of 48 byte */
#define SMALL_B1_BUFF_SIZE  240
#define LARGE_B1_BUFF_SIZE  4080

#define TXSEG_FIXED         2	  /* transmit segment descriptors	*/
#define RXSEG_FIXED         3	  /* receive segment descriptors	*/

#define NUM_TXSEG  TXSEG_FIXED    /* transmit segment descriptors per tpd
				     - we have only one segment		*/

/*
 * receive segment descriptors per rpd
 * we should better cover the worst case:
 *  The CP starts putting data being received in *ONE* small buffer.
 *  If data doesn't fit into only large buffers are used for the rest of it.
 */
#define NUM_RXSEG  (((MAX_PDU_LENGTH-SMALL_B1_BUFF_SIZE) \
		     +(LARGE_B1_BUFF_SIZE-1))/LARGE_B1_BUFF_SIZE)+1
										
/* number of additional segment descriptors */
#define TPD_EXTENSION   (NUM_TXSEG-TXSEG_FIXED)
#define RPD_EXTENSION   (((NUM_RXSEG-RXSEG_FIXED) | 3) + 1)

/* the size of a TPD/RPD must be a multiple of 32 bytes */
#define TPD_SIZE           (32*((TPD_EXTENSION/4)+1))
#define RPD_SIZE           (32*((RPD_EXTENSION/4)+1))
					
/* the size of the CP internal buffer pool */
#define SMALL_B1_POOL_SIZE	512
#define LARGE_B1_POOL_SIZE      512

#define SMALL_B1_NUM_RBUFFS	64
#define LARGE_B1_NUM_RBUFFS	64

#define SMALL_B1_LOW_WATER	16
#define LARGE_B1_LOW_WATER	16

/*
 * Resources for the init command.
 */
#define QSIZE_CMD               16      /* command queue capacity */
#define QSIZE_RX                32      /* receive queue capacity */
#define QSIZE_TX                128     /* transmit queue capacity */
#define QSIZE_RESUPPLY          4       /* buffer re-supply queue capacity */
#define RBD_BLK_SIZE        	32      /* descriptors per supply block */
#define NUM_CONNECTIONS         1024    /* number of ATM connections */

#define PCA200E_STAT_MEM	(32 * (QSIZE_CMD + \
				       QSIZE_TX + \
				       QSIZE_RX + \
				       QSIZE_RESUPPLY*2) + \
				 QSIZE_TX*TPD_SIZE + \
				 QSIZE_RX*RPD_SIZE + \
				 QSIZE_RESUPPLY*RBD_BLK_SIZE + \

#define PCA200E_DEV(d)		((struct pca200e_dev *) (d)->dev_data)
#define MSECS(n)		(n*HZ/1000)+1
#define PCA200E_INIT_TIMEOUT	MSECS(300)	/* maximum polling duration in
					           ms during
						   initialization */
#define PCA200E_COMMAND_TIMEOUT	MSECS(400)	/* maximum polling duration in
						   ms */

enum Status
{
  stat_pending	= 0x01U,	/* initial status (written by host)	*/
  stat_complete	= 0x02U,	/* completion status (written by cp) 	*/
  stat_free	= 0x04U,	/* initial status (written by host)	*/
  stat_error	= 0x08U		/* completion status (written by cp)	*/
};
typedef enum Status fw_status_t;

#define STATUS(x)		((volatile unsigned)(*x->status))

#define SET_STATUS(x, s)       	(STATUS(x) = s)
#define STAT_FREE(x)		(STATUS(x) == stat_free)
#define STAT_COMPLETE(x)	(STATUS(x) & stat_complete)
#define STAT_ERROR(x)		(STATUS(x) & stat_error)


/* poll expression <x> until it is <s> for at most <d> jiffies and
   return (x == s) */
#define POLL_STATUS(x,s,d) \
({ \
  int __poll_i = jiffies;  \
  while (jiffies < __poll_i + d) \
    if (x == s) \
      break; \
  (x == s); \
})

/* poll the status member of x until it is stat_complete[|stat_error]
   for at most <d> jiffies and return TRUE on error */
#define POLL_STAT_COMPLETE(x,d) \
({ \
  int __poll_i = jiffies; \
  while (jiffies < __poll_i + d) \
    if (STAT_COMPLETE(x)) \
      break; \
  STAT_ERROR(x) | !STAT_COMPLETE(x); \
})

/*
 * Command opcodes.
 */
enum Opcodes {
  op_initialize		= 0x01U,	/* init 200-series		*/
  op_activate_vcin,			/* activate incoming VCI	*/
  op_activate_vcout,			/* activate outgoing VCI	*/
  op_deactivate_vcin,			/* deactivate incoming VCI	*/
  op_deactivate_vcout,			/* deactivate outgoing VCI	*/
  op_request_stats,			/* return AAL and buffer stats	*/
  op_oc3_set_reg,			/* modify OC3 register		*/
  op_oc3_get_reg,			/* return OC3 register value	*/
#if FIRMWARE_VERSION >= 410 /* < FT4.1 firmware */
  op_zero_stats,			/* zero card statistics */
#endif
  op_get_prom_data,			/* return expansion rom info */
  op_setvpi_bits,			/* set x bits of those decoded by*/
					/* the firmware to be low order */
					/* bits from the VPI field of the*/
					/* ATM cell header. */
  op_interrupt_sel     = 0x80U
					/* interrupt select */
};
typedef volatile enum Opcodes fw_opcode_t;

/*
 * Activate VCI; used by the host to enable the reassembly of cells with the
 * specified VCI. Op word format:
 *
 * 31     24 23    16 15      8 7       0
 * +--------+--------+---------+--------+
 * |reserved| bscheme|     aal | opcode |
 * +--------+--------+---------+--------+
 */
struct Activate_block
{
  fw_opcode_t	op;
  unsigned	vpvc;
  unsigned	mtu;
};
typedef volatile struct Activate_block activate_block_t;

/*
 * Deactivate VCI command block.
 */
struct Deactivate_block
{
  fw_opcode_t	op;
  unsigned	vpvc;
};
typedef volatile struct Deactivate_block deactivate_block_t;

/*
 * Request statistics command block.
 */
struct Request_stats {
  fw_opcode_t	op;
  unsigned	stats_buff;     /* DMA address of stats buffer */
};
typedef volatile struct Request_stats request_stats_t;


/*
 * Expansion ROM Data Structure (as DMAed back)
 */
struct Expansion_rom_data
{
    unsigned		hw_version;
    unsigned		serial_number;
    unsigned char	mac_addr[8];
};
typedef volatile struct Expansion_rom_data expansion_rom_data_t;

/*
 * prom_data_buff is the physical address of the expansion_rom_data_t
 *
 * op is set to op_get_prom_data
 */
struct Request_prom_data
{
  fw_opcode_t	op;
  unsigned	prom_data_buff; /* DMA address of prom_data buffer */
}; 
typedef volatile struct Request_prom_data request_prom_data_t;

/*
 * OC3 SET/GET REG: used by the host to alter specific SUNI register values,
 * or return register contents to the host.
 *
 * op structure:
 *
 * 31     24 23    16 15      8 7       0
 * +--------+--------+---------+--------+
 * |  mask  | value  |   reg   | opcode |
 * +--------+--------+---------+--------+
 *
 * 'opcode' is the OC3 register operation to be performed, either reading, or
 * writing.
 *
 * 'reg' is the SUNI OC3 register to operate on.
 *
 * 'mask' specifes which bits within 'value' should be interpreted when writing
 * to th SUNI reg.
 *
 * 'value' specifies the logical value to place at each bit position of 'reg'
 * that 'mask' says should be interpreted.
 *
 * SUNI write example: Set the two high order bits of SUNI reg 0x34 to 0, not
 * altering the low order 6 bits.
 *
 * mask = 0xC0 (high order 2 bits set to 1) value = 0x00 (high order 2 bits set
 * to 0) reg = 0x34 opcode = op_oc3_set_reg
 *
 * (i.e. Only consider the two high order bits of 'value' when writing to
 * register 0x34 leaving the other 6 bits alone, and the value of these two
 * bits are 0)
 *
 * SUNI read example: Return the value from SUNI reg 0x34.
 *
 * reg = 0x34 opcode = op_oc3_get_reg
 *
 * The value of register 0x34 is returned to the host.
 *
 */
struct Oc3_reg_op
{
  fw_opcode_t	oc3_op;
  unsigned	oc3_buff;       /* DMA address of stats buffer */
};
typedef volatile struct Oc3_reg_op oc3_reg_op_t;

union Command
{
  fw_opcode_t		op;
  activate_block_t	activate_param;
  deactivate_block_t	deactivate_param;
  request_stats_t	stats_param;
  request_prom_data_t	prom_data_param;
  oc3_reg_op_t		oc3_reg_param;
  unsigned		force_multiple4[4]; /* gcc960 does this */
};
typedef volatile union Command command_t;

struct Command_block
{
  command_t	cmd;
  unsigned 	stataddr;
  unsigned	force_multiple4[3]; /* gcc960 does this */
};
typedef volatile struct Command_block qcard_cmd_t;

/*
 * Queue entry for transmit, receive and supply queues.
 */
struct Queue_entry
{
  unsigned	ioblock;	/* host DMA address of i/o block	*/
  unsigned	stataddr;	/* host DMA address of comletion status	*/
};
typedef volatile struct Queue_entry qcard_t;

/*
 * Initialize command block:
 */
struct Bscheme_specification
{
  unsigned	queue_length;
  unsigned	buffer_size;	/* host buffer size			*/
  unsigned	pool_size;	/* number of Rbd's			*/
  unsigned	supply_blksize;	/* num of Rbd's in ioblock (multiple
                                 * of 4 between 4 and 124 inclusive)	*/
};
typedef volatile struct Bscheme_specification bspec_t;

struct Init_block
{
  fw_opcode_t	op;
  fw_status_t	status;			/* initialized and read directly by host */
  unsigned	receive_threshold;	/* not used */
  unsigned	num_connect;		/* ATM connections */
  unsigned	cqueue_len;		/* command queue */
  unsigned	tqueue_len;		/* transmit queue */
  unsigned	rqueue_len;		/* receive queue */
  unsigned	rpd_extension;		/* number of 32 byte blocks */
  unsigned	tpd_extension;		/* number of 32 byte blocks */
  unsigned	conless_vpvc;		/* connectionless VPI/VCI */
  unsigned	pad[2];			/* force quad alignment */
  bspec_t	small_b1;		/* buffer scheme 1, small */
  bspec_t	large_b1;		/* buffer scheme 1, large */
  bspec_t	small_b2;		/* buffer scheme 2, small */
  bspec_t	large_b2;		/* buffer scheme 2, large */
};
typedef volatile struct Init_block init_block_t;

/*
 * Shared read-only structure used by host to initialize host resident queue
 * pointers.
 *
 * Each queue is write only. To initiate an operation the host writes a queue
 * entry with an address of the input/output block located in host memory.
 */
struct Cp_queues
{
    unsigned	command_queue;
    unsigned    transmit_queue;
    unsigned	receive_queue;
    unsigned	small_b1_queue;
    unsigned	large_b1_queue;
    unsigned	small_b2_queue;
    unsigned	large_b2_queue;
    unsigned	imask;            /* non zero enables CP to host interrupts */
    unsigned	istat;            /* 1 for interrupt posted		    */
    unsigned	heap_base;        /* offset form beginning of ram	    */
    unsigned	heap_size;	  /* space in bytes available for queues    */
    unsigned	hlogger;          /* non zero for host logging		    */
    unsigned	heartbeat;
    unsigned	firmware_release;
    unsigned	mon960_release;
    unsigned	tq_plen;          /* transmit throughput measurements       */

    /*
     * To be compatible with host compilers make sure the init block remains
     * on a quad word boundary. gcc960 does this.
     */
    init_block_t init;            /* one time command, not in cmd queue     */
    unsigned	media_type;       /* media type id			    */
    unsigned	oc3_revision;     /* OC3 revision number		    */
};
typedef volatile struct Cp_queues cp_queues_t;


struct transmit_segment
{
    unsigned	buffer;		/* transmit buffer DMA address	*/
    unsigned	length;		/* number of bytes in buffer	*/
};
typedef volatile struct transmit_segment tsd_t;

/*
 * ATM header definition:
 *  (without HEC-byte)
 *  31 28 27    20 19           4 3 1  0
 *  +----+--------+--------------+---+---+
 *  | gfc|   vpi  |      vci     |plt|clp|
 *  +----+--------+--------------+---+---+
 *
 */
struct atm_header
{
  unsigned clp  : 1;	/* cell loss priority	*/
  unsigned plt	: 3;	/* payload type		*/
  unsigned vci  :16;
  unsigned vpi  : 8;
  unsigned gfc  : 4;	/* generic flow control	*/
};
typedef struct atm_header atm_header_t;


/*
 * Tsd specification (spec) word format:
 *
 * 31 28   24 23    16 15     8 7      0
 * +----+----+--------+--------+--------+
 * |intr| aal| num seg|     PDU len     |
 * +----+----+--------+--------+--------+
 *
*/
struct tpd_spec {
  unsigned pdu_len:16;
  unsigned num_seg: 8;
  unsigned aal    : 4;
  unsigned intr   : 4;
};
typedef struct tpd_spec tpd_spec_t;

/*
 * Tsd specification (spec) word format:
 *
 * 31                16 15                 0
 * +----+----+----+----+----+----+----+----+
 * |     data cells    |    empty cells    |
 * +----+----+----+----+----+----+----+----+
 *
*/
struct tpd_str_desc {
  unsigned empty_cells:16;
  unsigned data_cells :16;
};
typedef struct tpd_str_desc tpd_str_desc_t;

/*
 * Transmit ioblock member encoding (the tpd dma address written to the transmit
 * queue).
 *
 * 31                   5 4 3 2 1 0
 * +---------------------+-+-------+
 * | ioblock addr        |R|     N |
 * ---------------------------------
 *
 * R = reserved (zero) N = size of Tpd in 8 word blocks
 */
struct ioblock
{
  unsigned tpd_size: 4;
  /* the rest is not needed */
};
typedef struct ioblock ioblock_t;


struct tx_pd
{
  atm_header_t		atm_header;
  tpd_spec_t		spec;
  tpd_str_desc_t	rc_stream_desc;   /* Number of data cells/interval */
  unsigned		pad;
  tsd_t	        	segment[NUM_TXSEG];
};
typedef volatile struct tx_pd tpd_t;


typedef volatile struct qhost_rbuff qhost_rbuff_t;   /* forward - see below */
struct receive_segment
{
  unsigned     	handle;		/* receive buffer handle */
  unsigned	length;		/* number of bytes in buffer */
};
typedef volatile struct receive_segment rsd_t;

struct rx_pd
{
    atm_header_t	atm_header;
    unsigned		nseg;
    rsd_t		segment[NUM_RXSEG];
};
typedef volatile struct rx_pd rpd_t;


enum buffer_magnitude
{
  bmag_small = 0x01,
  bmag_large = 0x02
};
typedef volatile enum buffer_magnitude buffer_magn_t;

struct receive_buffer_descriptor
{
  unsigned	handle;	  /* host CPU control address of buffer */
  unsigned     	buffer;	  /* DMA address of host buffer */
};
typedef volatile struct receive_buffer_descriptor rbd_t;




typedef volatile struct qhost_cmd qhost_cmd_t;
struct qhost_cmd
{
  qhost_cmd_t*	  next;
  qcard_cmd_t*	  qentry;	/* pointer to card queue entry		*/
  fw_status_t*	  status;	/* pointer to host resident status word	*/
  void *	  ioblock;      /* pointer to host resident ioblock	*/
};

typedef volatile struct qhost_tx qhost_tx_t;
struct qhost_tx
{
  qhost_tx_t*		next;
  qcard_t*		qentry;	/* pointer to card queue entry		*/
  fw_status_t*		status;	/* pointer to host resident status word	*/
  tpd_t*		tpd;	/* pointer to host resident ioblock	*/
  struct sk_buff*	skb;	/* skb assoc. with this tx queue entry  */
  struct atm_vcc*	vcc;	/* vcc assoc. with this tx queue entry  */
  void*                 txbuff; /* used in case of misaligned data      */
};

typedef volatile struct qhost_rx qhost_rx_t;
struct qhost_rx
{
  qhost_rx_t*      next;
  qcard_t*         qentry;	/* pointer to card queue entry */     
  fw_status_t*     status;	/* pointer to host resident status word */
  rpd_t*           rpd;		/* pointer to host resident ioblock */
};

typedef volatile struct qhost_supp qhost_supp_t;
struct qhost_supp
{
  qhost_supp_t*   next;
  qcard_t*        qentry;	/* pointer to card queue entry */
  fw_status_t*    status;	/* pointer to host resident status word */
  rbd_t*          rbd;		/* pointer to host resident ioblock */
};

/* typedef volatile struct qhost_rbuff qhost_rbuff_t; */
/* already declared above - see rsd_t */
struct qhost_rbuff
{
  qhost_rbuff_t*	next;
  buffer_magn_t		type;
  void*       		buff;
};

/*
 * Statistics.
 */
struct Phy_4b5b_stats200_t {
  unsigned    crc_header_errors; /* cells received with bad header CRC */
  unsigned    framing_errors;    /* cells received with bad framing */
  unsigned    pad[2];            /* pad to gcc960 boundary */
};
typedef volatile struct Phy_4b5b_stats200_t phy_4b5b_stats200_t;

struct Atm_stats200_t		/* ATM layer */
{
  unsigned    cells_transmitted; /* cells transmitted */
  unsigned    cells_received;    /* cells received */
  unsigned    vpi_bad_range;     /* cell drops: VPI out of range */
  unsigned    vpi_no_conn;       /* cell drops: no connection for VPI */
  unsigned    vci_bad_range;     /* cell drops: VCI out of range */
  unsigned    vci_no_conn;       /* cell drops: no connection for VCI */
  unsigned    pad[2];            /* pad to gcc960 boundary */
};
typedef volatile struct Atm_stats200_t atm_stats200_t;

struct Oc3_stats200_t		/* OC3 PHY */
{
  unsigned    section_bip8_errors;/* section 8 bit interleaved parity */
  unsigned    path_bip8_errors;  /* path 8 bit interleaved parity */
  unsigned    line_bip24_errors; /* line 24 bit interleaved parity */
  unsigned    line_febe_errors;  /* line far end block errors */
  unsigned    path_febe_errors;  /* path far end block errors */
  unsigned    corr_hcs_errors;   /* correctable header check sequence */
  unsigned    ucorr_hcs_errors;  /* uncorrectable header check sequence */
  unsigned    pad[1];                /* pad to gcc960 boundary */
};
typedef volatile struct Oc3_stats200_t oc3_stats200_t;

struct Aal0_stats200_t	/* Null AAL */
{
  unsigned    cells_transmitted;     /* cells transmitted */
  unsigned    cells_received;        /* cells received */
  unsigned    cells_dropped;         /* cells dropped */
  unsigned    pad[1];                /* pad to gcc960 boundary */
};
typedef volatile struct Aal0_stats200_t aal0_stats200_t;

struct Aal4_stats200_t	/* AAL4 */
{
  unsigned    cells_transmitted;     /* cells xmitted from segmented PDUs */
  unsigned    cells_received;        /* cells reassembled into PDUs */
  unsigned    cells_crc_errors;      /* payload CRC error count */
  unsigned    cells_protocol_errors; /* SAR or CS layer protocol errors */
  unsigned    cells_dropped;         /* cells dropped: partial reassembly */
  unsigned    cspdus_transmitted;    /* CS PDUs transmitted */
  unsigned    cspdus_received;       /* CS PDUs received */
  unsigned    cspdus_protocol_errors;/* CS layer protocol errors */
  unsigned    cspdus_dropped;        /* reassembled PDUs drop'd (in cells) */
  unsigned    pad[3];                /* pad to gcc960 boundary */
};
typedef volatile struct Aal4_stats200_t aal4_stats200_t;

struct Aal5_stats200_t	/* AAL5 */
{
  unsigned    cells_transmitted; /* cells transmitted from segmented SDUs */
  unsigned    cells_received;    /* cells reassembled into SDUs */
  unsigned    cells_dropped;     /* reassembled PDUs dropped (in cells) */
  unsigned    congestion_experienced; /* CRC error and length wrong */
  unsigned    cspdus_transmitted;/* CS PDUs transmitted */
  unsigned    cspdus_received;   /* CS PDUs received */
  unsigned    cspdus_crc_errors; /* Protocol Data Unit CRC errors */
  unsigned    cspdus_protocol_errors; /* CS layer protocol errors */
  unsigned    cspdus_dropped;    /* reassembled PDUs dropped */
  unsigned    pad[3];            /* pad to gcc960 boundary */
};
typedef volatile struct Aal5_stats200_t aal5_stats200_t;

struct Auxiliary_stats200_t
{
  unsigned    small_b1_failed;     /* receive BD allocation failures */
  unsigned    large_b1_failed;     /* receive BD allocation failures */
  unsigned    small_b2_failed;     /* receive BD allocation failures */
  unsigned    large_b2_failed;     /* receive BD allocation failures */
  unsigned    rpd_alloc_failed;    /* receive PDU allocation failures */
  unsigned    receive_carrier;     /* no carrier = 0, carrier = 1 */
  unsigned    pad[2];            /* pad to gcc960 boundary */
};
typedef volatile struct Auxiliary_stats200_t auxiliary_stats200_t;

struct Oc3_reg_set200_t {
  unsigned   reg[128];
};
typedef volatile struct Oc3_reg_set200_t oc3_reg_set200_t;

struct Stats200_t
{
  phy_4b5b_stats200_t		phy4b5b;
  oc3_stats200_t		phyoc3;
  atm_stats200_t		atm;
  aal0_stats200_t		aal0;
  aal4_stats200_t		aal4;
  aal5_stats200_t		aal5;
  auxiliary_stats200_t	aux200;
};
typedef volatile struct Stats200_t stats200_t;


typedef enum
{
  cold_start     = (unsigned) 0xc01dc01d,
  self_test_ok   = (unsigned) 0x02201958,
  self_test_fail = (unsigned) 0xadbadbad,
  cp_running     = (unsigned) 0xce11feed,
  mon960_too_big = (unsigned) 0x10aded00
} boot_status_t;

typedef volatile struct
{
  unsigned send;
  unsigned recv;
} soft_uart_t;

typedef volatile struct
{
  soft_uart_t	uart;		/* accessed by mon960 and ncomm */
  boot_status_t	bstat;		/* status */
  unsigned	app_base;	/* application base offset */
  unsigned	version;	/* mon960 version */
} monitor_t;

typedef struct
{
  unsigned cookie;
  unsigned firmwareid;
  unsigned loadoffset;
  unsigned startoffset;
} firmheader_t;




enum cardstate_t
{
  cs_blank,		/* the virgin card			*/
  cs_map,		/* mapped into virtual memory		*/
  cs_reset,		/* reset through bcr			*/
  cs_loadfirm,		/* firmware loaded			*/
  cs_startfirm,		/* ... and started			*/
  cs_init,		/* init command successful		*/
  cs_init_cmd,		/* command queue init ok		*/
  cs_init_tx,		/* transmit queue init ok		*/
  cs_init_rx,		/* receive queue init ok		*/
  cs_init_sb1,		/* small buffer queue init ok		*/
  cs_init_lb1,		/* large buffer queue init ok		*/
  cs_int,		/* interrupt requested and granted	*/
  cs_complete		/* destination unreachable		*/
};



struct pca200e_dev
{
  unsigned char		name[11];
  unsigned char		bus;
  unsigned char		dev_fn;
  unsigned char		irq;	
  unsigned long		phys_rambase;
  unsigned long		virt_rambase;

  struct pca200e_dev *	next;
  struct atm_dev *	atmdev;

  cp_queues_t*		cpqueue;
  unsigned char		mac[ESI_LEN];
  
  qhost_cmd_t*		cmd_queue;
  unsigned long		cmd_status_page;
  unsigned long		cmding;
  
  qhost_tx_t*		tx_queue;	/* here we insert PDUs for sending   */
  qhost_tx_t*		tx_queue_tail;	/* points to the first completed PDU */
  struct wait_queue*	tx_wait_queue;
  unsigned long		txing;
  unsigned long		tx_status_page;
  unsigned long		tpd_page;
  unsigned long		available_cell_rate;

  qhost_rx_t*		rx_queue;
  unsigned long		rx_status_page;
  unsigned long		rpd_page;
  
  qhost_supp_t*		smlb1_queue;
  unsigned long		smlb1_status_page;
  unsigned long		smlb1_rbd_page;
  qhost_rbuff_t*        smlb1_rbuff_head;
  unsigned              smlb1_count;

  qhost_supp_t*		lrgb1_queue;
  unsigned long		lrgb1_status_page;
  unsigned long		lrgb1_rbd_page;
  qhost_rbuff_t*        lrgb1_rbuff_head;
  unsigned		lrgb1_count;
  
  enum cardstate_t	cardstate;	/* for take-off and touch-down */
  qhost_rbuff_t*	bufferlist[SMALL_B1_NUM_RBUFFS+LARGE_B1_NUM_RBUFFS+1];
};

struct pca200e_dump
{
  struct pca200e_dev	dev;
  cp_queues_t		cpqueue;
  monitor_t		monitor;
  stats200_t		stats;
};

#endif


